@@ -27,6 +27,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.URI;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -112,6 +113,7 @@ public class PDFBoxAdapterTestCase {
     protected static final String TTSubset7 = "ttsubset7.pdf";
     protected static final String TTSubset8 = "ttsubset8.pdf";
     protected static final String TTSubset9 = "ttsubset9.pdf";
+    protected static final String TTSubset10 = "ttsubset10.pdf";
     protected static final String CFFCID1 = "cffcid1.pdf";
     protected static final String CFFCID2 = "cffcid2.pdf";
     protected static final String Type1Subset1 = "t1subset.pdf";
@@ -285,7 +287,7 @@ public class PDFBoxAdapterTestCase {
         os.reset();
         adapter.createStreamFromPDFBoxPage(doc, page, "key", pageAdjust, null, r);
         pdfdoc.outputTrailer(os);
-        Assert.assertTrue(os.toString("UTF-8").contains("/Fields ["));
+        Assert.assertTrue(os.toString(StandardCharsets.UTF_8.name()).contains("/Fields ["));
         doc.close();
     }
 
@@ -319,7 +321,7 @@ public class PDFBoxAdapterTestCase {
         adapter.createStreamFromPDFBoxPage(doc, page, "key", pageAdjust, null, r);
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         pdfdoc.output(os);
-        String out = os.toString("UTF-8");
+        String out = os.toString(StandardCharsets.UTF_8.name());
         Assert.assertTrue(out.contains("/Parent "));
         Assert.assertTrue(out.contains("/Kids "));
         doc.close();
@@ -342,7 +344,7 @@ public class PDFBoxAdapterTestCase {
         adapter.createStreamFromPDFBoxPage(doc, page, "key", pageAdjust, null, r);
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         pdfdoc.outputTrailer(os);
-        Assert.assertTrue(os.toString("UTF-8").contains("/Fields []"));
+        Assert.assertTrue(os.toString(StandardCharsets.UTF_8.name()).contains("/Fields []"));
         doc.close();
     }
 
@@ -361,7 +363,7 @@ public class PDFBoxAdapterTestCase {
         adapter.createStreamFromPDFBoxPage(doc, page, "key", pageAdjust, null, r);
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         pdfdoc.outputTrailer(os);
-        Assert.assertTrue(os.toString("UTF-8").contains("/Fields []"));
+        Assert.assertTrue(os.toString(StandardCharsets.UTF_8.name()).contains("/Fields []"));
         doc.close();
     }
 
@@ -403,13 +405,13 @@ public class PDFBoxAdapterTestCase {
         doc.close();
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         pdfdoc.output(bos);
-        Assert.assertFalse(bos.toString("UTF-8").contains("/W 5 /H 5 /BPC 8 /CS /RGB ID ÿÿÿ"));
+        Assert.assertFalse(bos.toString(StandardCharsets.UTF_8.name()).contains("/W 5 /H 5 /BPC 8 /CS /RGB ID ÿÿÿ"));
     }
 
     @Test
     public void testPSPDFGraphics2D() throws Exception {
         ByteArrayOutputStream stream = pdfToPS(IMAGE);
-        Assert.assertEquals(countString(stream.toString("UTF-8"), "%AXGBeginBitmap:"), 1);
+        Assert.assertEquals(countString(stream.toString(StandardCharsets.UTF_8.name()), "%AXGBeginBitmap:"), 1);
 
         pdfToPS(CFF1);
         pdfToPS(CFF2);
@@ -421,7 +423,7 @@ public class PDFBoxAdapterTestCase {
         pdfToPS(TTSubset3);
         pdfToPS(TTSubset5);
         stream = pdfToPS(CFFCID1);
-        Assert.assertEquals(countString(stream.toString("UTF-8"), "%AXGBeginBitmap:"), 2);
+        Assert.assertEquals(countString(stream.toString(StandardCharsets.UTF_8.name()), "%AXGBeginBitmap:"), 2);
         pdfToPS(CFFCID2);
         pdfToPS(Type1Subset1);
         pdfToPS(Type1Subset2);
@@ -431,7 +433,7 @@ public class PDFBoxAdapterTestCase {
         pdfToPS(LINK);
         pdfToPS(LOOP);
         stream = pdfToPS(LIBREOFFICE);
-        Assert.assertTrue(stream.toString("UTF-8").contains("/MaskColor [ 255 255 255 ]"));
+        Assert.assertTrue(stream.toString(StandardCharsets.UTF_8.name()).contains("/MaskColor [ 255 255 255 ]"));
 
     }
 
@@ -484,19 +486,15 @@ public class PDFBoxAdapterTestCase {
     @Test
     public void testSmask() throws IOException, ImageException {
         ByteArrayOutputStream ps = pdfToPS(SMASK);
-        Assert.assertTrue(ps.toString("UTF-8").contains("/Pattern"));
-        Assert.assertTrue(ps.toString("UTF-8").contains("{<\nf1f1f1"));
+        Assert.assertTrue(ps.toString(StandardCharsets.UTF_8.name()).contains("/Pattern"));
+        Assert.assertTrue(ps.toString(StandardCharsets.UTF_8.name()).contains("{<\nf1f1f1"));
     }
 
     @Test
-    public void testPCL() throws IOException, ImageException {
-        String ex = "";
-        try {
-            pdfToPCL(SHADING);
-        } catch (UnsupportedOperationException e) {
-            ex = e.getMessage();
-        }
-        Assert.assertTrue(ex.contains("Clipping is not supported."));
+    public void testPCL() {
+        UnsupportedOperationException ex = Assert.assertThrows(UnsupportedOperationException.class, () ->
+                pdfToPCL(SHADING));
+        Assert.assertTrue(ex.getMessage().contains("Clipping is not supported."));
     }
 
     private void pdfToPCL(String pdf) throws IOException, ImageException {
@@ -577,7 +575,7 @@ public class PDFBoxAdapterTestCase {
         PDFResources res = c.getPage().getPDFResources();
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         res.output(bos);
-        Assert.assertTrue(bos.toString("UTF-8").contains("/ExtGState << /GS1"));
+        Assert.assertTrue(bos.toString(StandardCharsets.UTF_8.name()).contains("/ExtGState << /GS1"));
     }
 
     @Test
@@ -640,29 +638,20 @@ public class PDFBoxAdapterTestCase {
     }
 
     @Test
-    public void testErrorMsgToPS() throws IOException, ImageException {
-        String msg = "";
-        try {
-            pdfToPS(ERROR);
-        } catch (RuntimeException e) {
-            msg = e.getMessage();
-        }
-        Assert.assertTrue(msg.startsWith("Error while painting PDF page: " + ERROR));
+    public void testErrorMsgToPS() {
+        RuntimeException ex = Assert.assertThrows(RuntimeException.class, () -> pdfToPCL(ERROR));
+        Assert.assertTrue(ex.getMessage().startsWith("Error while painting PDF page: " + ERROR));
     }
 
     @Test
-    public void testErrorMsgToPDF() throws IOException {
-        String msg = "";
+    public void testErrorMsgToPDF() {
         PDFDocument pdfdoc = new PDFDocument("");
         PDFContentGenerator contentGenerator = new PDFContentGenerator(pdfdoc, null, null);
         PDFRenderingContext context = new PDFRenderingContext(null, contentGenerator, null, null);
         ImagePDF imagePDF = new ImagePDF(new ImageInfo(ERROR, null), null);
-        try {
-            new PDFBoxImageHandler().handleImage(context, imagePDF, null);
-        } catch (RuntimeException e) {
-            msg = e.getMessage();
-        }
-        Assert.assertTrue(msg.startsWith("Error on PDF page: " + ERROR));
+        RuntimeException ex = Assert.assertThrows(RuntimeException.class, () ->
+                new PDFBoxImageHandler().handleImage(context, imagePDF, null));
+        Assert.assertTrue(ex.getMessage().startsWith("Error on PDF page: " + ERROR));
     }
 
     @Test
@@ -695,18 +684,14 @@ public class PDFBoxAdapterTestCase {
 
     @Test
     public void testMergeFontsAndFormXObject() throws IOException {
-        String msg = "";
         PDDocument doc = load(IMAGE);
         PDPage page = doc.getPage(0);
         AffineTransform pageAdjust = new AffineTransform();
-        try {
+        RuntimeException ex = Assert.assertThrows(RuntimeException.class, () ->
             getPDFBoxAdapter(true, true)
-                    .createStreamFromPDFBoxPage(doc, page, IMAGE, pageAdjust, new FontInfo(), new Rectangle());
-        } catch (RuntimeException e) {
-            msg = e.getMessage();
-        }
+                    .createStreamFromPDFBoxPage(doc, page, IMAGE, pageAdjust, new FontInfo(), new Rectangle()));
         doc.close();
-        Assert.assertEquals(msg, "merge-fonts and form-xobject can't both be enabled");
+        Assert.assertEquals(ex.getMessage(), "merge-fonts and form-xobject can't both be enabled");
     }
 
     @Test
@@ -719,7 +704,7 @@ public class PDFBoxAdapterTestCase {
         doc.close();
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         formXObject.output(bos);
-        Assert.assertTrue(bos.toString("UTF-8").contains("/Type /XObject"));
+        Assert.assertTrue(bos.toString(StandardCharsets.UTF_8.name()).contains("/Type /XObject"));
     }
 
     @Test
@@ -746,7 +731,7 @@ public class PDFBoxAdapterTestCase {
         pdfdoc.setFilterMap(filterMap);
         pdfdoc.output(os);
         doc.close();
-        return os.toString("UTF-8");
+        return os.toString(StandardCharsets.UTF_8.name());
     }
 
     @Test
@@ -761,7 +746,7 @@ public class PDFBoxAdapterTestCase {
         loadPage(pdfdoc, IMAGE);
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         pdfdoc.output(bos);
-        Assert.assertTrue(bos.toString("UTF-8").contains("/Filter /DCTDecode"));
+        Assert.assertTrue(bos.toString(StandardCharsets.UTF_8.name()).contains("/Filter /DCTDecode"));
     }
 
     @Test
@@ -790,7 +775,8 @@ public class PDFBoxAdapterTestCase {
             TTFParser parser = new TTFParser(true);
             TrueTypeFont trueTypeFont = parser.parse(inputStream);
             int i = 0;
-            for (GlyphData glyphData : trueTypeFont.getGlyph().getGlyphs()) {
+            for (int gid = 0; gid < trueTypeFont.getNumberOfGlyphs(); gid++) {
+                GlyphData glyphData = trueTypeFont.getGlyph().getGlyph(gid);
                 if (glyphData != null && glyphData.getDescription().isComposite()) {
                     compositeList.add(i);
                 }
@@ -806,7 +792,7 @@ public class PDFBoxAdapterTestCase {
         loadPage(pdfdoc, FORMROTATED);
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         pdfdoc.output(bos);
-        Assert.assertFalse(bos.toString("UTF-8").contains("/R 90"));
+        Assert.assertFalse(bos.toString(StandardCharsets.UTF_8.name()).contains("/R 90"));
     }
 
     @Test
@@ -846,7 +832,7 @@ public class PDFBoxAdapterTestCase {
     private String getAnnotationsID(PDFPage page) throws IOException {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         page.getAnnotations().output(os);
-        return os.toString("UTF-8").split("\n")[1];
+        return os.toString(StandardCharsets.UTF_8.name()).split("\n")[1];
     }
 
     private PDFPage drawAnnot(PDFDocument pdfDoc, Map<String, Object> usedFields, String pdf) throws Exception {
@@ -876,7 +862,7 @@ public class PDFBoxAdapterTestCase {
         Assert.assertEquals(getAnnotationsID(page1), id1);
         String id2 = "32 0 R";
         Assert.assertEquals(getAnnotationsID(page2), id2);
-        String outStr = os.toString("UTF-8").replaceAll("\\s\\s/", "/");
+        String outStr = os.toString(StandardCharsets.UTF_8.name()).replaceAll("\\s\\s/", "/");
         Assert.assertTrue(outStr.contains("<< /Kids [31 0 R] /T ([Signer1) >>"));
         Assert.assertTrue(outStr.contains("<<\n"
                 + "/Kids [" + id1 + " " + id2 + "]\n"
@@ -895,7 +881,7 @@ public class PDFBoxAdapterTestCase {
         PDFPage page = loadPage(pdfdoc, doc, new Rectangle());
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         page.getPDFResources().output(bos);
-        Assert.assertTrue(bos.toString("UTF-8").contains("/Properties << /S /S >>"));
+        Assert.assertTrue(bos.toString(StandardCharsets.UTF_8.name()).contains("/Properties << /S /S >>"));
     }
 
     @Test
@@ -906,7 +892,7 @@ public class PDFBoxAdapterTestCase {
         loadPage(pdfdoc, doc, destRect);
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         pdfdoc.output(bos);
-        String outStr = bos.toString("UTF-8").replaceAll("\\s\\s/", "/");
+        String outStr = removeWhiteSpace(bos);
         Assert.assertTrue(outStr.contains("<<\n"
                 + "/Type /Pattern\n"
                 + "/PatternType 2\n"
@@ -914,4 +900,35 @@ public class PDFBoxAdapterTestCase {
                 + "/Matrix [53.8858833313 0 0 -26.4968185425 72.7459411621 -20.8601989746]\n"
                 + ">>"));
     }
+
+    @Test
+    public void testPatternMatrixFormXObject() throws Exception {
+        PDDocument doc = load(SHADING);
+        PDFDocument pdfdoc = new PDFDocument("");
+        pdfdoc.setFormXObjectEnabled(true);
+        Rectangle destRect = new Rectangle(0, 1650, 274818, 174879);
+        loadPage(pdfdoc, doc, destRect);
+        ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        pdfdoc.output(bos);
+        String outStr = removeWhiteSpace(bos);
+        Assert.assertTrue(outStr.contains("/Pattern << /Pa1 12 0 R /Pa2 13 0 R >>"));
+        Assert.assertTrue(outStr.contains("<<\n"
+                + "/Type /Pattern\n"
+                + "/PatternType 2\n"
+                + "/Shading 2 0 R\n"
+                + "/Matrix [120 0 0 -120 162 705]\n"
+                + ">>"));
+    }
+
+    private String removeWhiteSpace(ByteArrayOutputStream bos) throws Exception {
+        return bos.toString(StandardCharsets.UTF_8.name()).replaceAll("\\s\\s/", "/");
+    }
+
+    @Test
+    public void testAscenderDoesntMatch() throws IOException {
+        FontInfo fi = new FontInfo();
+        writeText(fi, TTSubset6);
+        String msg = writeText(fi, TTSubset7);
+        Assert.assertTrue(msg, msg.contains("/C2_0745125721 12 Tf"));
+    }
 }
